/*
 * Copyright 1999-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * $Id: ProcessorStylesheetDoc.java,v 1.7 2004/08/17 18:18:08 jycli Exp $
 */
package org.apache.xalan.processor;

/**
 * This class processes the xsl:stylesheet element.  At the 
 * moment, it defers all methods to it's superclass.
 * @xsl.usage internal
 */
public class ProcessorStylesheetDoc extends XSLTElementProcessor
{
    static final long serialVersionUID = -1661497592836231844L;
}
